@@ -3,7 +3,6 @@ import json
 
 nativetracking = ["sonos", "xiaomi", "apple", "windows", "huawei", "samsung", "alexa", "roku"]
 nativetrackinghex = ["736f6e6f73","7869616f6d69","6170706c65","77696e646f7773","687561776569","73616d73756e67","616c657861","726f6b75"]
-
 siteshex = {
 "tiktok":"74696b746f6b",
 "tinder":"74696e646572",
@@ -11,7 +10,7 @@ siteshex = {
 "snapchat":"736e617063686174",
 "instagram":"696e7374616772616d",
 "fortnite":"6570696367616d65732f666f72746e6974652f686f6d65",
-"message":"6d657373656e676572",
+"messenger":"6d657373656e676572",
 "leagueoflegends":"6c65616775656f666c6567656e6473",
 "9gag":"39676167",
 "tumblr":"74756d626c72",
@@ -63,8 +62,12 @@ class NoCredentials(Exception):
         super().__init__(self.message)
 
 class FailedCredentials(Exception):
-    def __init__(self, message="Credentials in account.login() function failed. Please check credentials and login using account.login(email,password)"):
-        self.message = message
+    def __init__(self, error):
+        self.error = error
+        if error == """{"errors":{"code":"invalid"}}""":
+            self.message = f"2FA code invalid. Please check credentials, login using account.login(email,password) and enter the correct 2FA code"
+        else:    
+            self.message = f"Credentials in account.login() function failed. Please check credentials and login using account.login(email,password)\nError: {error}"
         super().__init__(self.message)
 
 class OptionUnavailable(Exception):
@@ -74,9 +77,9 @@ class OptionUnavailable(Exception):
         super().__init__(self.message)
 
 class ConfigNotFound(Exception):
-    def __init__(self, config, message="Supplied config cannot be found, probably cause it does not exist"):
+    def __init__(self, config):
         self.config = config
-        self.message = message
+        self.message = f"Config {config} cannot be found, probably cause it does not exist"
         super().__init__(self.message)
 
 class account:
@@ -90,8 +93,12 @@ class account:
                 login = requests.post('https://api.nextdns.io/accounts/@login', headers=headers, json=json)
                 if login.text == "OK":
                     success = 1
+                elif login.text == """{"requiresCode":true}""":
+                    code = input("""Please enter 2FA Code: """)
+                    json = {"email":f"{email}","password":f"{password}","code":f"{code}"}
+                    login = requests.post('https://api.nextdns.io/accounts/@login', headers=headers, json=json)
                 else:
-                    raise FailedCredentials
+                    raise FailedCredentials(login.text)
             c = login.cookies.get_dict()
             c = c['pst']
             headers['Cookie'] = f'pst={c}'
@@ -112,21 +119,30 @@ class settings:
             return list
     def setup(config,header):
         setup = requests.get(f"https://api.nextdns.io/configurations/{config}/setup", headers=header)
-        setup = setup.json()
-        return setup
+        if setup.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            setup = setup.json()
+            return setup
     def clearlogs(config,header):
         logs = requests.delete(f"https://api.nextdns.io/configurations/{config}/logs", headers = header)
         if logs.text == "Not Found":
             raise ConfigNotFound(config)
+        else:
+            return logs.text
     def rename(name, config, header):
-        name = {"name":name}
-        rename = requests.patch(f"https://api.nextdns.io/configurations/{config}/settings", headers = header, json=name)
+        nname = {"name":name}
+        rename = requests.patch(f"https://api.nextdns.io/configurations/{config}/settings", headers = header, json=nname)
         if rename.text == "Not Found":
             raise ConfigNotFound(config)
+        else:
+            return f"Config renamed to {name}"
     def delete(config,header):
         dconfig = requests.delete(f"https://api.nextdns.io/configurations/{config}", headers = header)
         if dconfig.text == "Not Found":
             raise ConfigNotFound(config)
+        else:
+            return f"Config {config} deleted"
     def logclientips(bool, config, header):
         if bool == True:
             bool = "true"
@@ -136,6 +152,8 @@ class settings:
         logcips = requests.patch(f"https://api.nextdns.io/configurations/{config}/settings", headers = header, json=logcips)
         if logcips.text == "Not Found":
             raise ConfigNotFound(config)
+        else:
+            return logcips.text
     def logdomains(bool, config, header):
         if bool == True:
             bool = "true"
@@ -145,6 +163,8 @@ class settings:
         logdom = requests.patch(f"https://api.nextdns.io/configurations/{config}/settings", headers = header, json=logdom)
         if logdom.text == "Not Found":
             raise ConfigNotFound(config)
+        else:
+            return logdom.text
     def blockpage(bool,config, header):
         if bool == True:
             bool = "true"
@@ -154,11 +174,16 @@ class settings:
         bp = requests.patch(f"https://api.nextdns.io/configurations/{config}/settings", headers = header, json=bp)
         if bp.text == "Not Found":
             raise ConfigNotFound(config)
+        else:
+            return bp.text
     def updatelinkedip(config,header):
         r = settings.setup(config, header)
         updatetoken = r["linkedIpUpdateToken"]
         updateip = requests.get(f"https://link-ip.nextdns.io/{config}/{updatetoken}")
-        return updateip.text
+        if updateip.text != "Bad Request":
+            raise ConfigNotFound(config)
+        else:
+            return updateip.text
 
 class security:
     def list(config, header):
@@ -178,7 +203,7 @@ class security:
         if setting.text == "Not Found":
             raise ConfigNotFound(config)
         else:
-            return settings
+            return setting.text
     def aidetection(bool, config, header):
         if bool == True:
             bool = "true"
@@ -189,7 +214,7 @@ class security:
         if setting.text == "Not Found":
             raise ConfigNotFound(config)
         else:
-            return settings
+            return setting.text
     def safebrowsing(bool, config, header):
         if bool == True:
             bool = "true"
@@ -200,7 +225,7 @@ class security:
         if setting.text == "Not Found":
             raise ConfigNotFound(config)
         else:
-            return settings
+            return setting.text
     def cryptojacking(bool, config, header):
         if bool == True:
             bool = "true"
@@ -211,7 +236,7 @@ class security:
         if setting.text == "Not Found":
             raise ConfigNotFound(config)
         else:
-            return settings
+            return setting.text
     def dnsrebinding(bool, config, header):
         if bool == True:
             bool = "true"
@@ -222,7 +247,7 @@ class security:
         if setting.text == "Not Found":
             raise ConfigNotFound(config)
         else:
-            return settings
+            return setting.text
     def homograph(bool, config, header):
         if bool == True:
             bool = "true"
@@ -233,7 +258,7 @@ class security:
         if setting.text == "Not Found":
             raise ConfigNotFound(config)
         else:
-            return settings
+            return setting.text
     def typosquatting(bool, config, header):
         if bool == True:
             bool = "true"
@@ -244,7 +269,7 @@ class security:
         if setting.text == "Not Found":
             raise ConfigNotFound(config)
         else:
-            return settings
+            return setting.text
     def dga(bool, config, header):
         if bool == True:
             bool = "true"
@@ -255,7 +280,7 @@ class security:
         if setting.text == "Not Found":
             raise ConfigNotFound(config)
         else:
-            return settings
+            return setting.text
     def newdomains(bool, config, header):
         if bool == True:
             bool = "true"
@@ -266,7 +291,7 @@ class security:
         if setting.text == "Not Found":
             raise ConfigNotFound(config)
         else:
-            return settings
+            return setting.text
     def parked(bool, config, header):
         if bool == True:
             bool = "true"
@@ -277,7 +302,7 @@ class security:
         if setting.text == "Not Found":
             raise ConfigNotFound(config)
         else:
-            return settings
+            return setting.text
     def csam(bool, config, header): 
         if bool == True:
             bool = "true"
@@ -288,15 +313,23 @@ class security:
         if setting.text == "Not Found":
             raise ConfigNotFound(config)
         else:
-            return settings
+            return setting.text
     def addtld(tld, config, header):
         hex = tld.encode('utf-8').hex()
         hex = f"hex:{hex}"
         put = requests.put(f"https://api.nextdns.io/configurations/{config}/security/blocked_tlds/{tld}", headers = header)
+        if put.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            return put.text
     def removetld(tld, config, header):
         hex = tld.encode('utf-8').hex()
         hex = f"hex:{hex}"
         remove = requests.delete(f"https://api.nextdns.io/configurations/{config}/security/blocked_tlds/{tld}", headers = header)
+        if remove.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            return remove.text
 
 class privacy:
     def list(config, header):
@@ -316,7 +349,7 @@ class privacy:
         if setting.text == "Not Found":
             raise ConfigNotFound(config)
         else:
-            return "OK"
+            return setting.text
     def blockaffiliate(bool, config, header):
         if bool == True:
             bool = "false"
@@ -327,7 +360,7 @@ class privacy:
         if setting.text == "Not Found":
             raise ConfigNotFound(config)
         else:
-            return "OK"
+            return setting.text
     def blocknative(native, config, header):
         hex = native.encode('utf-8').lower().hex()
         
@@ -352,13 +385,11 @@ class privacy:
 class parental:
     def list(config, header):
         settings = requests.get(f"https://api.nextdns.io/configurations/{config}/parentalcontrol", headers = header)
-        sites = requests.get(f"https://api.nextdns.io/configurations/{config}/parentalcontrol/services/@all")
-        if settings.text == "Not Found" or sites.text == "Not Found":
+        if settings.text == "Not Found":
             raise ConfigNotFound(config)
         else:
             settings = settings.json()
-            sites = sites.json()
-            return [settings, sites]
+            return settings
     def porn(bool, config, header):
         if bool == True:
             setting = requests.put(f"https://api.nextdns.io/configurations/{config}/parentalcontrol/categories/hex:706f726e", headers = header)
@@ -415,6 +446,17 @@ class parental:
             raise ConfigNotFound(config)
         else:
             return settings
+    def youtubeRestrictedMode(bool, config, header):
+        if bool == True:
+            bool = "true"
+        else:
+            bool = "false"
+        setting = {"youtubeRestrictedMode":bool}
+        setting = requests.patch(f"https://api.nextdns.io/configurations/{config}/parentalcontrol", headers = header, json=setting)
+        if setting.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            return setting.text
     def blockbypass(bool, config, header):
         if bool == True:
             bool = "true"
@@ -425,25 +467,24 @@ class parental:
         if setting.text == "Not Found":
             raise ConfigNotFound(config)
         else:
-            return settings
+            return setting.text
     def blocksite(site, config, header):
         try:
             sitehex = siteshex[f"{site.lower()}"]
         except:
-            raise OptionUnavailable(allowed=siteshex)
-        
-        hex = f"hex:{sitehex}"
-        
-        put = requests.put(f"https://api.nextdns.io/configurations/{config}/parentalcontrol/services/{hex}", headers = header)
-        return "OK"
+            raise OptionUnavailable(allowed=site)
+        put = requests.put(f"https://api.nextdns.io/configurations/{config}/parentalcontrol/services/hex:{sitehex}", headers = header)
+        if put.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            return put.text
     def unblocksite(site, config, header):
         sitehex = siteshex[f"{site.lower()}"]
-        hex = f"hex:{sitehex}"
-        delete = requests.delete(f"https://api.nextdns.io/configurations/{config}/parentalcontrol/services/{hex}", headers = header)
+        delete = requests.delete(f"https://api.nextdns.io/configurations/{config}/parentalcontrol/services/hex:{sitehex}", headers = header)
         if delete.text == "Not Found":
             raise ConfigNotFound(config)
         else:
-            return "OK"
+            return delete.text
 
 class denylist:
     def list(config, header):
@@ -455,14 +496,18 @@ class denylist:
             return list
     def blockdomain(domain, config, header):
         hex = domain.encode('utf-8').hex()
-        hex = f"hex:{hex}"
-        put = requests.put(f"https://api.nextdns.io/configurations/{config}/denylist/{hex}", headers = header)
-        return "OK"
+        put = requests.put(f"https://api.nextdns.io/configurations/{config}/denylist/hex:{hex}", headers = header)
+        if put.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            return put.json()
     def unblockdomain(domain, config, header):
         hex = domain.encode('utf-8').hex()
-        hex = f"hex:{hex}"
-        put = requests.delete(f"https://api.nextdns.io/configurations/{config}/denylist/{hex}", headers = header)
-        return "OK"
+        delete = requests.delete(f"https://api.nextdns.io/configurations/{config}/denylist/hex:{hex}", headers = header)
+        if delete.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            return delete.json()
 
 class allowlist:
     def list(config, header):
@@ -470,53 +515,84 @@ class allowlist:
         if settings.text == "Not Found":
             raise ConfigNotFound(config)
         else:
-            settings = settings.json()
-            return settings
+            return settings.json()
     def add(domain, config, header):
         hex = domain.encode('utf-8').hex()
         hex = f"hex:{hex}"
-        put = requests.put(f"https://api.nextdns.io/configurations/{config}/parentalcontrol/allowlist/{hex}", headers = header)
-        return "OK"
+        put = requests.put(f"https://api.nextdns.io/configurations/{config}/allowlist/hex:{hex}", headers = header)
+        if put.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            return put.json()
     def remove(domain, config, header):
         hex = domain.encode('utf-8').hex()
-        hex = f"hex:{hex}"
-        put = requests.delete(f"https://api.nextdns.io/configurations/{config}/parentalcontrol/allowlist/{hex}", headers = header)
-        return "OK"
+        delete = requests.delete(f"https://api.nextdns.io/configurations/{config}/allowlist/hex:{hex}", headers = header)
+        if delete.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            return delete.json()
 
 class analytics:
     def counter(config, header):
         count = requests.get(f"https://api.nextdns.io/configurations/{config}/analytics/counters", headers = header)
-        count = count.json()
-        return count
+        if count.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            count = count.json()
+            return count
     def topresolveddomains(config, header):
         top = requests.get(f"https://api.nextdns.io/configurations/{config}/analytics/top_domains/resolved", headers = header)
-        top = top.json()
-        return top
+        if top.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            top = top.json()
+            return top
     def topblockeddomains(config, header):
         top = requests.get(f"https://api.nextdns.io/configurations/{config}/analytics/top_domains/blocked", headers = header)
-        top = top.json()
-        return top
+        if top.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            top = top.json()
+            return top
     def toplists(config, header):
         top = requests.get(f"https://api.nextdns.io/configurations/{config}/analytics/top_lists", headers = header)
-        top = top.json()
-        return top
+        if top.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            top = top.json()
+            return top
     def topdevices(config, header):
         top = requests.get(f"https://api.nextdns.io/configurations/{config}/analytics/top_devices", headers = header)
-        top = top.json()
-        return top
+        if top.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            top = top.json()
+            return top
     def topclientips(config, header):
         top = requests.get(f"https://api.nextdns.io/configurations/{config}/analytics/top_client_ips", headers = header)
-        top = top.json()
-        return top
+        if top.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            top = top.json()
+            return top
     def toprootdomains(config, header):
         top = requests.get(f"https://api.nextdns.io/configurations/{config}/analytics/top_root_domains", headers = header)
-        top = top.json()
-        return top
+        if top.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            top = top.json()
+            return top
     def gafam(config, header):
         gafam = requests.get(f"https://api.nextdns.io/configurations/{config}/analytics/gafam", headers = header)
-        gafam = gafam.json()
-        return gafam
+        if gafam.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            top = gafam.json()
+            return top
     def trafficdest(config, header):
         top = requests.get(f"https://api.nextdns.io/configurations/{config}/analytics/traffic_destination_countries", headers = header)
-        top = top.json()
-        return top
+        if top.text == "Not Found":
+            raise ConfigNotFound(config)
+        else:
+            top = top.json()
+            return top
